/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.common.graph;

/**
 * Represents a node in a topology
 * 
 * FIXME (BR): are the attached objects really needed?
 * 
 * @author michael.stein
 */
public class Node implements Cloneable, GraphElement {
	/**
	 * The node identifier. Has to be unique.
	 */
	protected final Object id;

	/**
	 * FIXME (BR) is this really needed?
	 */
	protected Object obj;

	public Node(Object obj, Object id) {
		this.obj = obj;
		this.id = id;
	}
	
	/**
	 * @param id
	 *            unique node identifier
	 */
	public Node(Object id) {
		this.obj = null;
		this.id = id;
	}

	/**
	 * @return the attached object
	 */
	public Object getObj() {
		return obj;
	}
	
	/**
	 * Attaches an object to this Node
	 * 
	 * @param obj
	 *            the attached object
	 */
	public void setObj(Object obj) {
		this.obj = obj;
	}

	@Override
	public Object getId() {
		return id;
	}


	/**
	 * This is an equals() implementation that check equality of all attributes. This method is not used by hash maps etc.
	 */
	@Override
	public boolean equalsAttributes(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Node other = (Node) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	
	
	/**
	 * Be careful: This hashCode() implementation considers only the node identifier. Extend this class if you need different behavior
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	/**
	 * Be careful: This equals() implementation considers only the node idetnifier. Use equalsAttributes for a comparison of all attributes or extend this class if you need different behavior
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Node other = (Node) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}

	/**
	 * This clone method creates a new Node object, but uses the same instance
	 * for id and obj! Override this method if you need different behavior
	 */
	@Override
	public Node clone() {
		return new Node(obj, id);
	}
	
	@Override
	public String toString() {
		return String.valueOf(getId());
	}

}
